@@ -17,13 +17,14 @@ from tqdm import tqdm
 from .alternative_graph_traversal import enumerate_paths
 from .graph_processing import generate_reduced_graph
 from .graph_traversal import compute_all_paths_multitarget_dict
-from .pathway import analyze_paths
+from .pathway import analyze_paths, analyze_paths_with_intermediates
 
 __all__ = [
     'wrapper_explore',
     'wrapper_optimize',
     'wrapper_combine',
     'wrapper_pathway_enrichment',
+    'get_all_paths_validation',
 ]
 
 logger = logging.getLogger(__name__)
@@ -479,7 +480,7 @@ def wrapper_pathway_enrichment(
     simple_paths: bool,
     export: bool = True,
     output: Optional[str] = None,
-    genesets: List[Mapping[str, Iterable[str]]] = None,
+    genesets: Mapping[str, Iterable[str]] = None,
 ):
     """Conduct pathway enrichment on the paths.
 
@@ -533,9 +534,12 @@ def wrapper_pathway_enrichment(
             genesets=genesets,
         )
 
+        if summarized_results is None:
+            continue
+
         results[(source_node, target_node)] = (summarized_results, paths_summary, enrichment_results)
 
-        if export:
+        if export and summarized_results:
             # export_results
             logger.info(f'Exporting results for pair {source_node} - {target_node}')
             # TODO Fix the lmax later since now it is hacked
@@ -551,3 +555,111 @@ def wrapper_pathway_enrichment(
                 json.dump(paths_summary, file, indent=2)
 
     return results
+
+
+def wrapper_intermediates_nodes(
+    reduced_graph: DiGraph,
+    intermediate_nodes: List[str],
+    node2id: Mapping[int, str],
+    source_nodes: List[Any],
+    target_nodes: List[Any],
+    lmax: int,
+    simple_paths: bool,
+):
+    """Identify intermediate paths.
+
+    :param reduced_graph: directed graph
+    :param node2id: dict nodes to ids
+    :param intermediate_nodes: nodes names to be included in the paths
+    :param source_nodes: iterable with sources nodes (usually drugs)
+    :param target_nodes: iterable with target nodes (usually diseases)
+    :param lmax: maximum length of the path allowed
+    :param simple_paths: if true, only simple paths are calculated
+    :return: paths.
+    """
+    id2node = {
+        v: k
+        for k, v in node2id.items()
+    }
+
+    _target_nodes = [node2id[target_node] for target_node in target_nodes]
+
+    results = {}
+
+    for source_node, target_node in itt.product(source_nodes, target_nodes):
+
+        target_id = node2id[target_node]
+
+        # Calculate all paths between source and target
+        paths = enumerate_paths(
+            graph=reduced_graph,
+            source=node2id[source_node],
+            targets=[target_id],
+            lmax=lmax,
+            cycle_free=simple_paths,
+        )
+
+        # Skip if there are no paths
+        if not paths:
+            logger.warning(f'No paths between {source_node} and {target_node}')
+            continue
+
+        # Get summarized results for export
+        final_paths, nodes_present = analyze_paths_with_intermediates(
+            reduced_graph=reduced_graph,
+            paths=paths,
+            id2node=id2node,
+            intermediate_nodes=intermediate_nodes,
+        )
+
+        if final_paths is None:
+            continue
+
+        results[(source_node, target_node)] = (final_paths, nodes_present)
+
+    return results
+
+
+def get_all_paths_validation(
+    graph: DiGraph,
+    source: str,
+    target: str,
+    lmax: int,
+    simple_paths: bool,
+):
+    """Get all paths between source and target.
+
+    :param graph: directed graph
+    :param source_nodes: iterable with sources nodes (usually drugs)
+    :param target_nodes: iterable with target nodes (usually diseases)
+    :param lmax: maximum length of the path allowed
+    :param simple_paths: if true, only simple paths are calculated
+    :return: all paths between the source and target
+    """
+    _check_generic_input(graph, [source], [target])
+
+    # Get the reduced version of the graph and the node2id mapping
+    reduced_graph, node2id = generate_reduced_graph(graph, [target])
+
+    id2node = {
+        v: k
+        for k, v in node2id.items()
+    }
+
+    # Calculate all paths between source and target
+    all_paths = enumerate_paths(
+        graph=reduced_graph,
+        source=node2id[source],
+        targets=[node2id[target]],
+        lmax=lmax,
+        cycle_free=simple_paths,
+    )
+
+    # revert the node ids to names for simplicity
+    return [
+        [
+            id2node[node]
+            for node in path
+        ]
+        for path in all_paths
+    ]
